#include <stdio.h>
#include <math.h>
#include "pgm.h"
#include "edges.h"
#include "cells.h"

extern unsigned char *image;
unsigned char *edge_map[ORIENTATIONS];

double *feature_map[ORIENTATIONS];
char *contrast_map;

main(int argc, char* argv[])
{
  int width, height, max;
  int i, j, k, centre, surround;
  int size;
  char file_name[]="horiz20.240.pgm";

/*
  if(argc==2)
    strcpy(file_name, argv[1]);

  max=read_pgm(file_name, &image, &width, &height);
  size=width*height;

  Edges(width, height, size);

*/
  max=read_pgm("edge000.pgm", &edge_map[0], &width, &height);  
  max=read_pgm("edge030.pgm", &edge_map[1], &width, &height);  
  max=read_pgm("edge045.pgm", &edge_map[2], &width, &height);  
  max=read_pgm("edge060.pgm", &edge_map[3], &width, &height);  
  max=read_pgm("edge090.pgm", &edge_map[4], &width, &height);  
  max=read_pgm("edge120.pgm", &edge_map[5], &width, &height);  
  max=read_pgm("edge135.pgm", &edge_map[6], &width, &height);  
  max=read_pgm("edge150.pgm", &edge_map[7], &width, &height);  

  size=width*height;


  for (i=0; i<ORIENTATIONS; i++)
    feature_map[i]=(double *) calloc(4*size, sizeof(double));

  contrast_map=(char *) calloc(4*size,1);

/*  for (centre=4; centre<8; centre++)
  {
*/
    centre=7;
    surround=14;

/*    for (surround=centre+2; surround<centre*2.5; surround++)
      {
*/
	Cells(width, height, size, centre, surround);
	for (k=0; k<ORIENTATIONS; k++)
     	  for (j=0; j<size; j++)
	    feature_map[k][j]=0;
/*      }
/*  }
*/

}


void Cells(int width, int height, int size, 
	   int radius_centre, int radius)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  static char file_name[]="c4.s06.o0.pgm",
	      contrast_name[]="c_map.c6.s06.pgm";
  float weight[ORIENTATIONS];
  double cell[ORIENTATIONS], max; 
  double **gaus_c, **gaus_s;

  file_name[1]='0'+radius_centre;
  temp=radius;
  file_name[4]='0'+(temp/10);
  temp=temp%10;
  file_name[5]='0'+temp;
  file_name[8]='0';

  contrast_name[7]=file_name[1];
  contrast_name[10]=file_name[4];
  contrast_name[11]=file_name[5];

  gaus_c=(double **) calloc(2*radius_centre+1, sizeof(double*));
  for (i=0; i<2*radius_centre+1; i++)
    gaus_c[i]=(double *) calloc(2*radius_centre+1, sizeof(double));

  gaus_s=(double **) calloc(2*radius+1, sizeof(double*));
  for (i=0; i<2*radius+1; i++)
    gaus_s[i]=(double *) calloc(2*radius+1, sizeof(double));

  Calc_Weights(weight);

  Calc_Gaus(gaus_c, radius_centre);
  Calc_Gaus(gaus_s, radius);

  for (pixel_y=radius; pixel_y<height-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<ORIENTATIONS; i++)
      {
	cell[i]=convolve(pixel_x, pixel_y, i, width,
			 radius_centre, gaus_c);
	cell[i]=cell[i]-convolve(pixel_x, pixel_y, i, width, radius, 
			  	 gaus_s);
      }

      for(i=0; i<ORIENTATIONS; i++)
	for(j=0; j<ORIENTATIONS; j++)
	  feature_map[i][pixel_y*width+pixel_x]+=
				weight[(j-i+ORIENTATIONS)%ORIENTATIONS]*cell[j];


    }/* pixels */


  max=Mult_Norm_dbl(feature_map, size);

  for(i=0; i<ORIENTATIONS; i++)
  {
    write_dbl_pgm(file_name, feature_map[i], width, height, 255);
    (file_name[8])++;
  }

  Combine_Maps(feature_map, size);
  if (max > 0)
    write_pgm(contrast_name, contrast_map, width, height, 255);
  else
    printf("failed\n");
}


void Combine_Maps(double **feature_map, int size)
{
  int i, j;
  double max=0.;

  for(i=0; i<size; i++)
  {
    for (j=0; j<ORIENTATIONS; j++)
      if (feature_map[j][i]>max)
	max=feature_map[j][i];
    contrast_map[i]=(char)max;
    max=0.;
  }

  return;
}

void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.; 
  denom=ORIENTATIONS*ORIENTATIONS;
  if (ORIENTATIONS%2)
    denom=denom-1;

  for (i=1; i<=ORIENTATIONS/2; i++)
  {
    w[i]=(float)-4*i/denom;
    w[ORIENTATIONS-i]=w[i];
  }
}

    

void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/2.0);
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;

}




double convolve(int x, int y, int map, int width, int rad, double **gaus)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
	temp+=(double) edge_map[map][(y-i)*width+(x-j)] * 
		gaus[i+rad][j+rad];

  return temp;
}



double Mult_Norm_dbl(double **array, int size)
{
  int i;
  double t_max, max=0;

  for(i=0; i<ORIENTATIONS; i++)
    if ((t_max=Max_dbl(array[i], size))>max)
      max=t_max;

  for(i=0; i<ORIENTATIONS; i++)
    Norm_dbl(array[i], max, size);

  return max;
}




void Norm_dbl(double *array, double max, int size)
{
  int i;
  double mult_fac;

  if (max>0)
  {
    mult_fac=255./max;
    for(i=0; i<size; i++)
      array[i]=array[i]*mult_fac;
  }
  else
    for(i=0; i<size; i++)
      array[i]=0;
  return;
}
    


void Norm_char(char* map, int size)
{
  int i, j, max;
  float multFac;

  max=Max_char(map, size);

  if(max>0)
  {
    multFac=255.0/max;

    for(i=0; i<size; i++)
      map[i]=map[i]*multFac;
  }
}




int Max_int(int *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}

double Max_dbl(double *map, int size)
{
  int i;
  double max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}


int Max_char(char *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}




int write_dbl_pgm(char *file_name, double *map, int width, 
		  int height, int max)
{
  unsigned char c_map[height*width];
  int i;
  
  for(i=0; i<height*width; i++)
  {
    if (map[i]>0.)
      c_map[i]=(unsigned char) map[i];
    else
      c_map[i]=0;
  }

  write_pgm(file_name, c_map, width, height, max);
}
