#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"contrast.h"

#define LEVELSIZE 125
#define EDGES 4

void f_to_c(float *from, unsigned char *to, int size);
void c_to_f(unsigned char *from, float *to, int size);
void d_to_c(double *from, unsigned char *to, int size);
void norm_double(double *array, int size);

unsigned char *i_map;
unsigned char **e_map;
float *c_map;
int orientations;
int width;

main(int argc, char* argv[])
{
   unsigned char *char_map;
  int height, max;
  int size;
  int i;

  FILE *f_ptr1, *f_ptr2;

  /* Read image file */
  max=read_pgm(argv[1], &i_map, &width, &height);
  size=width*height;
 
  /* find edges */
  e_map = (unsigned char **) malloc(EDGES*sizeof(unsigned char *));
  for (i=0; i<EDGES; i++)
    e_map[i] = (unsigned char *) malloc(size);
  Edges(width, height, size, e_map, i_map);
   

  /* calculate contrast values */
  contrast();

  f_ptr1=fopen("temp1", "w");
  for (i=0; i<size; i++)
    fprintf(f_ptr1, "%g ", c_map[i]);
  fclose(f_ptr1);

  char_map = (unsigned char *) malloc (size);
  f_to_c(c_map, char_map, size);

  f_ptr2=fopen("temp2", "w");
  for (i=0; i<size; i++)
    fprintf(f_ptr2, "%d ", char_map[i]);
  fclose(f_ptr2);

  write_pgm("contrast.pgm", char_map, width, height, max);

} /* main() */

void f_to_c(float *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i]=(unsigned char)from[i];
}

void c_to_f(unsigned char *from, float *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i]=(float) from[i];
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}

void norm_double(double *array, int size)
{
  double max=0.0, factor;
  int i;

  for (i=0; i<size; i++)
    if (array[i]>max) max=array[i];

  factor=255.0/max;
  for (i=0; i<size; i++)
    array[i]*=factor;
}

void contrast()
{
/************************************************/
/*  UNCOMMENT FOR FINAL VERSION
/************************************************
/*  int width=pyramid.levelSize[0];
/************************************************/

  int size = width * width;

/*----------------------------------------------*/
/*	REMOVE FOR FINAL VERSION		*/
/*----------------------------------------------*/
  int c_radius=5;
  int s_radius=8;
  unsigned char *char_map;
  float **o_data, *i_data;

/*----------------------------------------------*/


  static double threshold;
  double max, max_inten;
  double **orient, *inten;
  double **gaus_c, **gaus_s;

  int i;

/*----------------------------------------------*/
/*	REMOVE FOR FINAL VERSION		*/
/*----------------------------------------------*/
  orientations=EDGES;
/*----------------------------------------------*/


  inten = (double *) malloc(size * sizeof(double));
  o_data = (float **) malloc(orientations * sizeof(float *));
  orient = (double **) malloc(orientations * sizeof(double *));

  for (i=0; i<orientations; i++)
  {
    orient[i] = (double *) calloc(size, sizeof(double));
  }

  c_map = (float *) malloc(size * sizeof(float));

/************************************************/
/*  UNCOMMENT FOR FINAL VERSION
/************************************************
/*  orientations=user.FEATURE_EDGES_ORIENTATIONS;
/************************************************/

  if (orientations==8) orientations=4;

/************************************************/
/*  UNCOMMENT FOR FINAL VERSION
/************************************************
/*  i_data = data[0].level[0][0].value;
/*
/*  for(i=0; i<orientations; i++)
/*    o_data[i] = data[pyramid.startEdgeHierarchy + i].level[0][0].value;
/************************************************/



/*----------------------------------------------*/
/*	REMOVE FOR FINAL VERSION		*/
/*----------------------------------------------*/
  i_data = (float *) malloc(size * sizeof(float));
  c_to_f(i_map, i_data, size);

  for (i=0; i<orientations; i++)
  {
    o_data[i] = (float *) malloc(size * sizeof(float));
    c_to_f(e_map[i], o_data[i], size);
  }
/*----------------------------------------------*/

  /* allocate memory for gaussians */
  gaus_c=(double **) calloc(2*c_radius+1, sizeof(double*));
  for (i=0; i<2*c_radius+1; i++)
    gaus_c[i]=(double *) calloc(2*c_radius+1, sizeof(double));

  gaus_s=(double **) calloc(2*s_radius+1, sizeof(double*));
  for (i=0; i<2*s_radius+1; i++)
    gaus_s[i]=(double *) calloc(2*s_radius+1, sizeof(double));


  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, c_radius);
  Calc_Gaus(gaus_s, s_radius);


  max=Orientation(width, c_radius, s_radius, gaus_c, gaus_s, o_data,
		  orient);
  max_inten=Intensity(width, c_radius, s_radius, gaus_c, gaus_s, i_data,
		      inten);


  if (max_inten > max) max = max_inten;

  if (threshold < 0) threshold = set_threshold(max, orient, inten);

  if (max > threshold)
  {
    combine_maps(max, c_map, orient, inten, size);

/************************************************/
/*  UNCOMMENT FOR FINAL VERSION
/************************************************
/*    combine_maps(max, i_data, orient, inten, size);
/*    clear_maps();
/************************************************/

  }

} /* contrast()*/


double set_threshold(double max, double **o_contrast, 
		     double *i_contrast)
{
	/****  arbitrary and temporary ****/
  return max*0.85;
} /* set_threshold() */


void combine_maps(double factor, float *values, double **o_contrast,
		  double *i_contrast, int size)
{
  double max;
  int i, j;

  printf("max=%g\n", factor);

  factor=255.0/factor;

  printf("factor=%g\n", factor);

  for (i=0; i<size; i++)
  {
    max=i_contrast[i];
    for (j=0; j<orientations; j++)
      if (o_contrast[j][i] > max)
	max=o_contrast[j][i];
    values[i] = (float) (max*factor);
  }
} /* combine_maps */

void clear_maps()
{
  int i, j;

/*
  for (i=1; i<pyramid.numHierarchies; i++)
    for (j=0; j<pyramid.levelSize[0]; j++)
      data[i].level[0][0].value[j]=0.0;
*/
} /* clear_maps() */

double Orientation(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s,
		   float **edges, double **orient_contrast)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  float weight[orientations];
  double cell[orientations];
  double max=0.0;

  Calc_Weights(weight);

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<orientations; i++)
      {
        cell[i]=Convolve(pixel_x, pixel_y, width,
                         center_radius, gaus_c, edges[i]);
        cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius,
                                 gaus_s, edges[i]);
      }

      for(i=0; i<orientations; i++)
      {
        for(j=0; j<orientations; j++)
	{
          orient_contrast[i][pixel_y*width+pixel_x] 
			+= weight[(j-i+orientations)%orientations]
			   * cell[j];
	}
        if (orient_contrast[i][pixel_y*width+pixel_x]>max)
	  max=orient_contrast[i][pixel_y*width+pixel_x];
      } /* for (i) */

    }/* for (pixels) */

  return max;

}/* Orientation()*/


double Intensity(int width, int center_radius, int radius, 
		 double **gaus_c, double **gaus_s,
		 float *values, double *inten_contrast)
{
  int i, j, pixel_x, pixel_y;
  double cell;
  double max=0.0;

  /* convolve image with difference of gaussians */
  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
  {
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      cell=Convolve(pixel_x, pixel_y, width, center_radius, gaus_c,
		    values);
      cell=cell-Convolve(pixel_x, pixel_y, width, radius, gaus_s, values);

      inten_contrast[pixel_y*width+pixel_x]=fabs(cell);

      if(inten_contrast[pixel_y*width+pixel_x]>max)
	max = inten_contrast[pixel_y*width+pixel_x];
    }/* for pixel_x */
  } /* for(pixel_y) */

  return max;

} /* Intensity()*/



void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/2.0);
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */

void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.;

  denom=orientations*orientations;
  if (orientations%2)
    denom=denom-1;
 

  for (i=1; i<=orientations/2; i++)
  {
    w[i]=(float)(-4*i)/denom;
    w[orientations-i]=w[i];
  }

} /* Calc_Weights() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                float *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */

