#include <stdio.h>
#include <math.h>
#include "tls.h"
#include "pgm.h"

void init();

int T[225], L[225];

main(int argc, char* argv[])
{
  int width, height, max;
  int i, j;
  int size;
  unsigned char *feature_map[2];
  unsigned char *image;
  char base[20], fname[35];

  max=read_pgm(argv[1], &image, &width, &height);
  size=width*height;

  TLs(width, height, size, feature_map, image);

  strncpy(base, argv[1], strlen(argv[1])-3);
  base[strlen(argv[1])-3]=0;
  sprintf(fname, "%sT.pgm", base);
  write_pgm(fname, feature_map[0], width, height, max);
  sprintf(fname, "%sL.pgm", base);
  write_pgm(fname, feature_map[1], width, height, max);
}

void TLs(int width, int height, int size, unsigned char **edge_map1,
		unsigned char *image)
{
  int i, j, edge;
  int radius=2;
  int *temp_map1[ORIENTATIONS];
  char f_name[20];


  for (i=0; i<ORIENTATIONS; i++)
  {
    temp_map1[i]=(int *) calloc(4*size, sizeof(int));
    edge_map1[i]=(unsigned char *) calloc(4*size, 1);
  }

  printf("size=%d\n", size);

  init();

  for (edge=0; edge<2; edge++)
  {
    for (i=radius; i<height-radius; i++)
    {
      for (j=radius; j<width-radius; j++)
      {
        temp_map1[edge][i*width+j]=
				(int)abs(Sobel(i, j, edge, width, image));
      }
    }
  }

  MultipleNormalize(size, edge_map1, temp_map1);

}

void init()
{
  int ind, row, col;
  FILE *tptr, *lptr; 

  for (ind=0; ind<225; ind++)
  {
    T[ind]=-11;
    L[ind]=-11;
  }

  for(col=3;col<=4; col++)
    for(row=3; row<=13; row++)
      L[row*15+col]=53;

  for(row=12; row<=13; row++)
    for(col=5; col<=13; col++)
      L[row*15+col]=53;

  for(row=3; row<=4; row++)
    for(col=5; col<=13; col++)
      T[row*15+col]=53;

  for(col=7; col<=8; col++)
    for(row=5; row<=13; row++)
      T[row*15+col]=53;

  tptr=fopen("T_mask.pgm", "w");
  lptr=fopen("L_mask.pgm", "w");
  
  for(row=0; row<15; row++)
  {
    for(col=0; col<15; col++)
    {
      fprintf(tptr, "%d ", T[row*15+col]);
      fprintf(lptr, "%d ", L[row*15+col]);
    }
    fprintf(tptr, "\n");
    fprintf(lptr, "\n");
  }

  fclose(tptr);
  fclose(lptr);
}


float Sobel(int i, int j, int hierarchy, int width, unsigned char *image)
{
  float temp;

  switch(hierarchy)
  {
    case 0: return Grad(i, j, width, T, image);
    case 1: return Grad(i, j, width, L, image);
  }     
}


float Grad(int i, int j, int width, int *val, unsigned char *image)
{
  float total=0;
  int x, y;

  i=i-7;
  j=j-7; 

  for(y=0; y<15; y++)
    for(x=0; x<15; x++)
      total+=(val[y*15+x]*image[(i+y)*width + (j+x)]);

  return total;
}




void MultipleNormalize(int size, unsigned char **e_map, int **t_map)
{
  int i;
  float currMax, maxVal = 0.0;
  
  for(i=0; i<2; i++)
  {
    currMax = Maximum(i, size, t_map);
    if (currMax > maxVal)
       maxVal=currMax;
  }

  for(i=0; i<2; i++)

    Normalize(i, maxVal, size, e_map, t_map);

  return;
}  



void Normalize (int edge, int maxValue, int size, unsigned char **e_map,
		int **temp_map)
{
  int i;
  float multFac;

  multFac=255.0/maxValue;

  for(i=0; i<size; i++)
    e_map[edge][i]=(unsigned char)(temp_map[edge][i]*multFac);
}



float Maximum(int edge, int size, int **temp_map)
{
  int i;
  float max=0;

  for (i=0; i<size; i++)
  {
    if (temp_map[edge][i]>max)
      max=temp_map[edge][i];
  }

  return max;
}
