#include <stdio.h>
#include <math.h>
#include "edge_test.h"
#include "pgm.h"

double matrix[18][19];
double orig[18][19];
double x[18]={1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};

main(int argc, char* argv[])
{
  FILE *data;
  int width, height, max;
  int i, j, k;
  int size;
  double *edge_map[18];
  unsigned char *image;
  char base[20], f_name[35], new_name[35];
  double avg;
  double result;
  double top=0;

  data=fopen("edges.data", "w");

  for(i=0; i<18; i++)
  {
    avg=0.0;
    for(j=0; j<=18; j++)
      matrix[i][j]=0.0;

    if(i<10)
      sprintf(f_name, "images/elements/20x20/0%d0.pgm", i);
    else
      sprintf(f_name, "images/elements/20x20/%d0.pgm", i);

    max=read_pgm(f_name, &image, &width, &height);
    size=width*height;

    Edges(width, height, size, image, i, &avg);

    fprintf(data, "%d, %d\n", i, (long) avg);

    free(image);
  }

  init_matrix();
  solve(x);
  x[0]=1;

  printf("norm[18]={");

  for(i=0; i<18; i++)
  {
    printf("\t\t%.10f,\n", x[i], x[i]);
    result=0;
    for(j=0; j<18; j++)
      result+=orig[i][j]*x[j];
    fprintf(data, "%d\n", (long) result);
  }

  fprintf(data, "\n");

  for(i=0; i<18; i++)
  {
    avg=0.0;
    for(j=0; j<18; j++)
      matrix[i][j]=0.0;

    if(i<10)
      sprintf(f_name, "images/elements/20x20/0%d0.pgm", i);
    else
      sprintf(f_name, "images/elements/20x20/%d0.pgm", i);

    max=read_pgm(f_name, &image, &width, &height);
    size=width*height;

    Edges(width, height, size, image, i, &avg);

    fprintf(data, "%d, %g\n", i, avg);

    free(image);
  }

  fclose(data);
}

void init_matrix()
{
  int i;

  memcpy(orig, matrix, 18*19*sizeof(double));

  for(i=0; i<18; i++)
  {
    matrix[i][18]=-matrix[i][0];
    matrix[i][0]=-1;
  }
}

void solve(double *X)
{
  int row, col, el;
  double factor;
  int degree=18;

  for(row=0; row<degree; row++)
  {
    for(col=0; col<row; col++)
    {
      factor=-matrix[row][col];
      for(el=col; el<=degree; el++)
	matrix[row][el]+=matrix[col][el]*factor;
    }

    if (!matrix[row][row])
    {
      printf("\n\nrow=%d\n", row);
      for(col=0; col<=degree; col++)    
	printf("%g ", matrix[col]);
      printf("\n");
      exit(-1);
    }

    for(col=degree; col>=row; col--)
      matrix[row][col]=matrix[row][col]/matrix[row][row];
  }

/*
  printf("\n\nmatrix\n");
  for(row=0; row<degree; row++)
  {
    for(col=0; col<=degree; col++)
      printf("%g ", matrix[row][col]);
    printf("\n");
  }
*/
  for(row=degree-1; row>=0; row--)
  {
    for(col=row+1; col<degree; col++)
      matrix[row][degree]-=matrix[row][col]*X[col];
    X[row]=matrix[row][degree];
  }
}

void Edges(int width, int height, int size, unsigned char *image, 
		int orien, double *avg)
{
  int i, j, edge;
  int radius=1;
  double temp_map[size];
  char f_name[20];
  int edges[size];
  double mag[size];

  for (i=0; i<size; i++)
    edges[i]=-1;

  for (i=radius; i<height-radius; i++)
  {
    for (j=radius; j<width-radius; j++)
    {
      edges[i*width+j]=Sobel(i, j, width, image, mag);
    }
  }

  create_map(edges, temp_map, size, mag, orien*10);

  *avg=average(size, temp_map);

}

double average(int size, double *map)
{
  int i, count=0;
  double result=0.0;

  for(i=0; i<size; i++)
    if(map[i]>0.0)
    {
      result+=map[i];
      count++;
/*      printf("%g %g\n", map[i], result);*/
    } 

  return result;
}

void create_map(int *edges, double *map, int size, double *mag, 
		int orien)
{
  int focus, i;
  double diff, factor=255.0/90.0;

  focus=orien;
    for(i=0; i<size; i++)
    {
      if (edges[i]<0) map[i]=0.0;
      else
      {
	diff=fabs(focus-edges[i]);
	if (diff>90.0) diff=180-diff;
	map[i]=(90.0-diff)*mag[i]*x[(edges[i]+5)/10];
/*	printf("before: matrix[%d][%d]=%g\n", focus/10, (edges[i]+5)/10,
		matrix[focus/10][(edges[i]+5)/10]);*/
	matrix[focus/10][(edges[i]+5)/10]+=map[i];
/*	printf("edge: %d mag: %g map: %g\n", edges[i], mag[i], map[i]);
	printf("after: matrix[%d][%d]=%g\n\n", focus/10, (edges[i]+5)/10,
		matrix[focus/10][(edges[i]+5)/10]);*/
      }
    }
/*
  for(i=0; i<18; i++)
    printf("%g ", matrix[focus/10][i]);
  printf("\n");
*/  
}

int Sobel(int i, int j, int width, unsigned char *image, double *mag)
{
  float temp;
  double Sx, Sy, ratio;
  int result;

  static int X[9] = {-1,  0,  1,
		     -2,  0,  2,
		     -1,  0,  1 };

  static int Y[9] = {-1, -2, -1,
		      0,  0,  0,
		      1,  2,  1 };

  Sx=Grad(i, j, width, X, image);
  Sy=Grad(i, j, width, Y, image);
  mag[i*width+j]=sqrt(Sx*Sx+Sy*Sy);

  if (Sy==0.0)
  {
    if (Sx==0.0)  result=-1;
    else result=90;
  }
  else
  {
    ratio=Sx/Sy;
    if ((result = atan(ratio)*180/M_PI) < 0)
      result+=180;
  }
/*
  if (result>=180)
    printf("ratio=%g, angle=%g, result=%d\n", ratio,
		atan(ratio)*180/M_PI, result);
*/

  return result;
}

double Grad(int i, int j, int width, int mask[9], unsigned char *image)
{
  int x, y;
  double total=0.0;
  int temp;

  i--; j--;

  for(y=0; y<3; y++)
    for(x=0; x<3; x++)
    {
      temp = mask[y*3+x] * image[(i+y)*width + (j+x)];
      total+=temp;
    }

  return total;

}



void Normalize (int size, double *temp_map)
{
  int i;
  double multFac, max=0.0;

  for(i=0; i<size; i++)
    if(temp_map[i]>max) max=temp_map[i];

  multFac=255.0/max;

  for(i=0; i<size; i++)
    temp_map[i]=(int) (temp_map[i]*multFac);
}
